"use strict";

var BioEntity = require("./BioEntity");
var Point = require("../canvas/Point");

// noinspection JSUnusedLocalSymbols
var logger = require('../../logger');

/**
 * Class representing alias data.
 *
 * @param javaObject
 *          object de-serialized ajax query to the server side
 */
function Alias(javaObject) {
  BioEntity.call(this, javaObject);
  if (javaObject.idObject !== undefined) {
    this.setId(javaObject.idObject);
  } else {
    this.setId(javaObject.id);
  }
  this.setModelId(javaObject.modelId);
  if (javaObject.bounds !== undefined) {
    this.setX(javaObject.bounds.x);
    this.setY(javaObject.bounds.y);
    this.setWidth(javaObject.bounds.width);
    this.setHeight(javaObject.bounds.height);
  }

  if (this._modelId === undefined) {
    throw new Error("ModelId is not defined for alias" + javaObject);
  }

  if (javaObject.name === undefined) {
    this.setIsComplete(false);
  } else {
    this.update(javaObject);
  }
}

Alias.prototype = Object.create(BioEntity.prototype);
Alias.prototype.constructor = Alias;

/**
 * Updates alias with full data information. This function should be called when
 * full information about alias is retrieved from server.
 *
 * @param javaObject
 *          object representing data from server side
 */
Alias.prototype.update = function (javaObject) {
  if (javaObject instanceof Alias) {
    if (javaObject.getName() === undefined) {
      return;
    }
    this.setDescription(javaObject.getDescription());
    this.setElementId(javaObject.getElementId());
    this.setType(javaObject.getType());
    this.setCharge(javaObject.getCharge());
    this.setSymbol(javaObject.getSymbol());
    this.setFullName(javaObject.getFullName());
    this.setAbbreviation(javaObject.getAbbreviation());
    this.setFormula(javaObject.getFormula());
    this.setName(javaObject.getName());
    this.setSynonyms(javaObject.getSynonyms());
    this.setFormerSymbols(javaObject.getFormerSymbols());
    this.setReferences(javaObject.getReferences());
    this.setOther(javaObject.getOther());
    this.setHierarchyVisibilityLevel(javaObject.getHierarchyVisibilityLevel());
    this.setComplexId(javaObject.getComplexId());
    this.setTransparencyLevel(javaObject.getTransparencyLevel());
    this.setLinkedSubmodelId(javaObject.getLinkedSubmodelId());
    this.setCompartmentId(javaObject.getCompartmentId());
    this.setDescription(javaObject.getDescription());

    this.setConstant(javaObject.getConstant());
    this.setBoundaryCondition(javaObject.getBoundaryCondition());
    this.setInitialAmount(javaObject.getInitialAmount());
    this.setInitialConcentration(javaObject.getInitialConcentration());

    this.setX(javaObject.getX());
    this.setY(javaObject.getY());
    this.setWidth(javaObject.getWidth());
    this.setHeight(javaObject.getHeight());

    this.setIsComplete(true);
  } else {
    if (javaObject.name === undefined) {
      return;
    }
    this.setDescription(javaObject.notes);
    this.setElementId(javaObject.elementId);
    this.setType(javaObject.type);
    this.setCharge(javaObject.charge);
    this.setSymbol(javaObject.symbol);
    this.setFullName(javaObject.fullName);
    this.setAbbreviation(javaObject.abbreviation);
    this.setFormula(javaObject.formula);
    this.setName(javaObject.name);
    this.setSynonyms(javaObject.synonyms);
    this.setFormerSymbols(javaObject.formerSymbols);
    this.setReferences(javaObject.references);
    this.setOther(javaObject.other);
    this.setHierarchyVisibilityLevel(javaObject.hierarchyVisibilityLevel);
    this.setComplexId(javaObject.complexId);
    this.setTransparencyLevel(javaObject.transparencyLevel);
    this.setLinkedSubmodelId(javaObject.linkedSubmodel);
    this.setCompartmentId(javaObject.compartmentId);

    this.setConstant(javaObject.constant);
    this.setBoundaryCondition(javaObject.boundaryCondition);
    this.setInitialAmount(javaObject.initialAmount);
    this.setInitialConcentration(javaObject.initialConcentration);


    if (javaObject.bounds !== undefined) {
      this.setX(javaObject.bounds.x);
      this.setY(javaObject.bounds.y);
      this.setWidth(javaObject.bounds.width);
      this.setHeight(javaObject.bounds.height);
    }
    this.setIsComplete(true);
  }
};

Alias.prototype.getCharge = function () {
  return this.charge;
};
Alias.prototype.setCharge = function (charge) {
  this.charge = charge;
};

Alias.prototype.getConstant = function () {
  return this._constant;
};
Alias.prototype.setConstant = function (constant) {
  this._constant = constant;
};

Alias.prototype.getBoundaryCondition = function () {
  return this._boundaryCondition;
};
Alias.prototype.setBoundaryCondition = function (boundaryCondition) {
  this._boundaryCondition = boundaryCondition;
};

Alias.prototype.getInitialAmount = function () {
  return this._initialAmount;
};
Alias.prototype.setInitialAmount = function (initialAmount) {
  this._initialAmount = initialAmount;
};

Alias.prototype.getInitialConcentration = function () {
  return this._initialConcentration;
};
Alias.prototype.setInitialConcentration = function (initialConcentration) {
  this._initialConcentration = initialConcentration;
};

Alias.prototype.getFormerSymbols = function () {
  return this.formerSymbols;
};

Alias.prototype.setFormerSymbols = function (formerSymbols) {
  this.formerSymbols = formerSymbols;
};

Alias.prototype.getX = function () {
  return this.x;
};

Alias.prototype.setX = function (x) {
  if (x !== undefined) {
    this.x = x;
  }
};

Alias.prototype.setY = function (y) {
  if (y !== undefined) {
    this.y = y;
  }
};

Alias.prototype.getY = function () {
  return this.y;
};

Alias.prototype.setElementId = function (elementId) {
  this._elementId = elementId;
};

Alias.prototype.getElementId = function () {
  return this._elementId;
};

Alias.prototype.setWidth = function (width) {
  if (width !== undefined) {
    this.width = width;
  }
};

Alias.prototype.setHeight = function (height) {
  if (height !== undefined) {
    this.height = height;
  }
};
Alias.prototype.getWidth = function () {
  return this.width;
};

Alias.prototype.getHeight = function () {
  return this.height;
};

Alias.prototype.getName = function () {
  return this.name;
};

Alias.prototype.setName = function (name) {
  this.name = name;
};

Alias.prototype.getFullName = function () {
  return this.fullName;
};

Alias.prototype.setFullName = function (fullName) {
  this.fullName = fullName;
};

Alias.prototype.getCompartmentId = function () {
  return this._compartmentId;
};

Alias.prototype.setCompartmentId = function (compartmentId) {
  if (compartmentId !== null) {
    this._compartmentId = compartmentId;
  }
};
Alias.prototype.getComplexId = function () {
  return this._complexId;
};

Alias.prototype.setComplexId = function (complexId) {
  if (complexId !== null) {
    this._complexId = complexId;
  }
};

Alias.prototype.getTransparencyLevel = function () {
  return this._transparencyLevel;
};

Alias.prototype.setTransparencyLevel = function (transparencyLevel) {
  this._transparencyLevel = transparencyLevel;
};

Alias.prototype.getCenter = function () {
  return new Point(this.getX() + this.getWidth() / 2, this.getY() + this.getHeight() / 2);
};

module.exports = Alias;
